'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getClient = exports.getLock = exports.defaultLockOptions = exports.authIsRequired = exports.setAuth0User = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.refreshUser = refreshUser;

var _auth0Lock = require('auth0-lock');

var _auth0Lock2 = _interopRequireDefault(_auth0Lock);

var _auth0Js = require('auth0-js');

var _auth0Js2 = _interopRequireDefault(_auth0Js);

var _reactRouterRedux = require('react-router-redux');

var _reduxActions = require('redux-actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var AUTH0_CLIENT_ID = process.env.AUTH0_CLIENT_ID;
var AUTH0_DOMAIN = process.env.AUTH0_DOMAIN;

var setAuth0User = exports.setAuth0User = (0, _reduxActions.createAction)('set auth0 user');
var authIsRequired = exports.authIsRequired = AUTH0_CLIENT_ID && AUTH0_DOMAIN;
var defaultLockOptions = exports.defaultLockOptions = {
  auth: {
    params: {
      scope: 'openid analyst offline_access'
    },
    redirect: false
  },
  closeable: false,
  autoclose: true
};
var getLock = exports.getLock = function getLock(lockOptions) {
  return new _auth0Lock2.default(AUTH0_CLIENT_ID, AUTH0_DOMAIN, lockOptions);
};
var getClient = exports.getClient = function getClient() {
  return new _auth0Js2.default({
    clientID: AUTH0_CLIENT_ID,
    domain: AUTH0_DOMAIN
  });
};

/**
 * Use on application mount when authentication is required
 */
function refreshUser(dispatch, overrideAuthIsRequired) {
  var localStorage = window.localStorage;
  if (authIsRequired || overrideAuthIsRequired) {
    (function () {
      var client = getClient();
      var userString = localStorage.getItem('user');
      var user = userString && JSON.parse(userString);
      if (user && user.refreshToken) {
        dispatch(setAuth0User(user));
        if (process.env.NODE_ENV !== 'development') {
          client.refreshToken(user.refreshToken, function (err, delegationResult) {
            if (err) {
              dispatch(setAuth0User(null));
              localStorage.setItem('user', null);
              dispatch((0, _reactRouterRedux.push)('/login'));
            } else {
              user.idToken = delegationResult.id_token;
              dispatch(setAuth0User(user));
              localStorage.setItem('user', (0, _stringify2.default)(user));
            }
          });
        }
      } else {
        dispatch((0, _reactRouterRedux.push)('/login'));
      }
    })();
  } else {
    throw new Error('Auth0 credentials not found!');
  }
}

//# sourceMappingURL=auth0.js