'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetchError = exports.fetchMultiple = exports.fetchAction = exports.decrementFetches = exports.incrementFetches = exports.FETCH_ERROR = exports.FETCH_MULTIPLE = exports.FETCH = exports.DECREMENT_FETCH = exports.INCREMENT_FETCH = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.middleware = middleware;

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

var _lodash = require('lodash.isobject');

var _lodash2 = _interopRequireDefault(_lodash);

var _reduxActions = require('redux-actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var INCREMENT_FETCH = exports.INCREMENT_FETCH = 'increment outstanding fetches';
var DECREMENT_FETCH = exports.DECREMENT_FETCH = 'decrement outstanding fetches';
var FETCH = exports.FETCH = 'fetch';
var FETCH_MULTIPLE = exports.FETCH_MULTIPLE = 'fetch multiple';
var FETCH_ERROR = exports.FETCH_ERROR = 'fetch error';

var incrementFetches = exports.incrementFetches = (0, _reduxActions.createAction)(INCREMENT_FETCH);
var decrementFetches = exports.decrementFetches = (0, _reduxActions.createAction)(DECREMENT_FETCH);
var fetchAction = exports.fetchAction = (0, _reduxActions.createAction)(FETCH);
var fetchMultiple = exports.fetchMultiple = (0, _reduxActions.createAction)(FETCH_MULTIPLE);
var fetchError = exports.fetchError = (0, _reduxActions.createAction)(FETCH_ERROR);

function middleware(store) {
  return function (next) {
    return function (action) {
      if (action.type === FETCH) {
        return store.dispatch(runFetchAction(action.payload, store.getState()));
      } else if (action.type === FETCH_MULTIPLE) {
        return store.dispatch(runFetchMultiple(action.payload, store.getState()));
      } else {
        return next(action);
      }
    };
  };
}

exports.default = fetchAction;

/**
 * Calls fetch, adds Auth and Content header if needed. Automatically parses content based on type.
 *
 * @returns Promise
 */

function runFetch(_ref, state) {
  var _this = this;

  var _ref$options = _ref.options,
      options = _ref$options === undefined ? {} : _ref$options,
      _ref$retry = _ref.retry,
      retry = _ref$retry === undefined ? false : _ref$retry,
      url = _ref.url;

  var isJSON = (0, _lodash2.default)(options.body);
  return (0, _isomorphicFetch2.default)(url, (0, _extends3.default)({}, options, {
    body: isJSON ? (0, _stringify2.default)(options.body) : options.body,
    headers: (0, _extends3.default)({}, createAuthorizationHeader(state), createContentHeader(isJSON), options.headers || {})
  })).then(checkStatus).then(createResponse).then(function () {
    var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(response) {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.t0 = retry;

              if (!_context.t0) {
                _context.next = 5;
                break;
              }

              _context.next = 4;
              return retry(response);

            case 4:
              _context.t0 = _context.sent;

            case 5:
              if (!_context.t0) {
                _context.next = 9;
                break;
              }

              _context.t1 = runFetch({ options: options, retry: retry, url: url }, state);
              _context.next = 10;
              break;

            case 9:
              _context.t1 = response;

            case 10:
              return _context.abrupt('return', _context.t1);

            case 11:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    }));

    return function (_x) {
      return _ref2.apply(this, arguments);
    };
  }());
}

function runFetchAction(_ref3, state) {
  var next = _ref3.next,
      _ref3$options = _ref3.options,
      options = _ref3$options === undefined ? {} : _ref3$options,
      _ref3$retry = _ref3.retry,
      retry = _ref3$retry === undefined ? false : _ref3$retry,
      url = _ref3.url;

  return [incrementFetches(), runFetch({ options: options, retry: retry, url: url }, state).then(function (response) {
    return [decrementFetches(), next(null, response)];
  }).catch(function (error) {
    return createErrorResponse(error).then(function (response) {
      return [decrementFetches(), fetchError(response), next(error, response)];
    });
  })];
}

/**
 * @returns Promise
 */

function runFetchMultiple(_ref4, state) {
  var fetches = _ref4.fetches,
      next = _ref4.next;

  return [incrementFetches(), _promise2.default.all(fetches.map(function (fetch) {
    return runFetch(fetch, state);
  })).then(function (responses) {
    return [decrementFetches(), next(null, responses)];
  }).catch(function (error) {
    return createErrorResponse(error).then(function (response) {
      return [decrementFetches(), fetchError(response), next(error, response)];
    });
  })];
}

function createAuthorizationHeader(state) {
  return state.user && state.user.idToken ? { Authorization: 'bearer ' + state.user.idToken } : {};
}

function checkStatus(res) {
  if (res.status >= 200 && res.status < 300) {
    return res;
  } else {
    throw res;
  }
}

function createContentHeader(isJSON) {
  return isJSON ? { 'Accept': 'application/json', 'Content-Type': 'application/json;charset=UTF-8' } : {};
}

function createErrorResponse(res) {
  return res.headers ? createResponse(res) : _promise2.default.resolve(res);
}

function createResponse(res) {
  return deserialize(res).then(function (value) {
    return {
      url: res.url,
      status: res.status,
      statusText: res.statusText,
      headers: res.headers,
      value: value
    };
  }).catch(function (err) {
    return {
      value: err
    };
  });
}

function deserialize(res) {
  var header = res.headers.get('Content-Type') + ' ' + res.headers.get('Content');
  if (header.indexOf('application/json') > -1) return res.json();
  if (header.indexOf('application/ld+json') > -1) return res.json();
  if (header.indexOf('application/octet-stream') > -1) return res.arrayBuffer();
  return res.text();
}

//# sourceMappingURL=fetch.js